// Title:   FeatureList.h
// Created: Thu Sep 25 15:07:31 2003
// Authors: Richard Keiser, Mark Pauly
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __FEATURELIST_H_
#define __FEATURELIST_H_

#include "../../../../Core/DataStructures/src/SurfelCollection.h"

typedef struct featurePoint {
	Vector3D		position;
	SurfelInterface *surfel1;
	SurfelInterface *surfel2;
	bool			isCovered;
} FeaturePoint;

/**
 * Class for a double linked list of feature points.
 * This class is used by the <code>CSG_Tool</code>
 */
class FeatureList {

public:

	/**
	 * creatures an empty feature least
	 */
	FeatureList ();
	/**
	 * destructor
	 */
	virtual ~FeatureList (); 
	/**
	 * returns the number of elements in the list
	 */
	unsigned int count () const;
	/**
	 * returns true if the list ist empty
	 */
	bool isEmpty () const;
	/**
	 * if auto-delete is enabled, removed feature points from the list are deleted.
	 * Auto-delete is disabled by default.
	 */
	void setAutoDeleteEnabled(bool enable);
	/**
	 * append a feature point at the end of the list
	 */
	void append (FeaturePoint* obj );
	/**
	 * prepend a feature point at the beginnig of the list
	 */
	void prepend (FeaturePoint* obj);
	/**
	 * insert a feature point after the current iterator position
	 */
	void insert (FeaturePoint* obj);
	/**
	 * remove the feature point at the current iterator position
	 */
	bool remove ();
	/**
	 * marks an element as deleted without deleting it
	 */
	bool setAsDeleted ();
	/**
	 * removes all elements from the list
	 */
	virtual void clear ();
	/**
	 * returns the feature point at the current iterator position
	 */
	FeaturePoint* current ();
	/**
	 * returns the first feature point in the list
	 */
	FeaturePoint* getFirst () const;
	/** 
	 * returns the last feature point in the list
	 */
	FeaturePoint* getLast () const;
	/**
	 * returns the next feature point in the list without chaning the position of the iterator
	 */
	FeaturePoint* getNext () const;
	/**
	 * returns the previous feature point in the list without chaning the position of the iterator
	 */
	FeaturePoint* getPrev () const;
	/**
	 * sets the iterator to the first feature point in the list and returns it
	 */
	FeaturePoint* first ();
	/**
	 * sets the iterator to the last feature point in the list and returns it
	 */
	FeaturePoint* last ();
	/**
	 * sets the iterator to the next feature point in the list and returns it
	 */
	FeaturePoint* next ();
	/**
	 * sets the iterator to the previous feature point in the list and returns it
	 */
	FeaturePoint* prev ();
	/**
	 * returns the feature point at the current iterator position. 
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* current (bool &isDeleted) const;
	/**
	 * returns the first feature point in the list
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* getFirst (bool &isDeleted) const;
	/**
	 * returns the next feature point in the list without chaning the position of the iterator
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* getLast (bool &isDeleted) const;
	/**
	 * returns the previous feature point in the list without chaning the position of the iterator
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* getNext (bool &isDeleted) const;
	/**
	 * returns the previous feature point in the list without chaning the position of the iterator
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* getPrev (bool &isDeleted) const;
	/**
	 * sets the iterator to the first feature point in the list and returns it
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* first (bool &isDeleted);
	/**
	 * sets the iterator to the last feature point in the list and returns it
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* last (bool &isDeleted);
	/**
	 * sets the iterator to the next feature point in the list and returns it
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* next (bool &isDeleted);
	/**
	 * sets the iterator to the previous feature point in the list and returns it
	 * If it is marked as deleted, <code>isDeleted</code> is set to <code>true</code>
	 */
	FeaturePoint* prev (bool &isDeleted);

private:

	typedef struct element {
		FeaturePoint*	obj;
		struct element*		next;
		struct element*		prev;
		bool				isDeleted;
	} Element;


	Element *firstObj;
	Element *lastObj;
	Element *currentObj;
	int nObjs;
	bool bAutoDelete;
};


#endif //__FEATURELIST_H_
